@@ -1,8 +1,10 @@
 """ A module for making images of laminations. """
 
+from __future__ import annotations
+
 from math import sin, cos, pi, ceil
-from typing import List, Tuple, Dict, Any, Set, Optional
 from queue import PriorityQueue
+from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 from PIL import Image, ImageDraw  # type: ignore
 
@@ -20,7 +22,7 @@ ZOOM_FRACTION = 0.9
 VERTEX_BUFFER = 0.2
 
 LAMINATION_COLOUR = "#555555"
-TRIANGLE_COLOURS = ["#b5b5b5", "#c0c0c0", "#c7c7c7", "#cfcfcf"]
+TRIANGLE_COLOURS = {"bw": ["#b5b5b5", "#c0c0c0", "#c7c7c7", "#cfcfcf"], "rainbow": ["hsl({}, 50%, 50%)".format(i) for i in range(0, 360, 10)]}
 
 
 def deduplicate(items: List[Edge]) -> List[Edge]:
@@ -47,7 +49,7 @@ def interpolate(A: Coord, B: Coord, r: float = 0.5) -> Coord:
     return add(A, B, r, 1 - r)
 
 
-def support(triangulation: "bigger.Triangulation[Edge]", edge: Edge) -> Tuple[Triangle, Triangle]:
+def support(triangulation: bigger.Triangulation[Edge], edge: Edge) -> Tuple[Triangle, Triangle]:
     """ Return the two triangles that support and edge. """
 
     a, b, c, d = triangulation.link(edge)
@@ -58,7 +60,7 @@ def support(triangulation: "bigger.Triangulation[Edge]", edge: Edge) -> Tuple[Tr
     return triangle1, triangle2
 
 
-def adjacent(triangulation: "bigger.Triangulation[Edge]", current: Triangle, side: int) -> Tuple[Triangle, int]:
+def adjacent(triangulation: bigger.Triangulation[Edge], current: Triangle, side: int) -> Tuple[Triangle, int]:
     """ Return the (other triangle, other side) which shares an edge with triangle[side]. """
 
     return next(
@@ -69,7 +71,7 @@ def adjacent(triangulation: "bigger.Triangulation[Edge]", current: Triangle, sid
     )
 
 
-def supporting_triangles(triangulation: "bigger.Triangulation[Edge]", edges: List[Edge]) -> Tuple[List[List[Triangle]], Set[Edge]]:
+def supporting_triangles(triangulation: bigger.Triangulation[Edge], edges: List[Edge]) -> Tuple[List[List[Triangle]], Set[Edge]]:
     """ Return a list of list of triangles that support these edges, grouped by connectedness, and a set of edges that in the interior. """
 
     position_index = dict((edge, index) for index, edge in enumerate(edges))
@@ -106,11 +108,11 @@ def supporting_triangles(triangulation: "bigger.Triangulation[Edge]", edges: Lis
     return ordered_components, interior
 
 
-def layout_triangulation(triangulation: "bigger.Triangulation[Edge]", components: List[List[Triangle]], interior: Set[Edge]) -> Dict[Triangle, FlatTriangle]:
-    """ Return a dictionary mapping the triangles that meet the given edges to coordinates in the plane.
+def default_layout_triangulation(triangulation: bigger.Triangulation[Edge], components: List[List[Triangle]], interior: Set[Edge]) -> Dict[Triangle, FlatTriangle]:
+    """Return a dictionary mapping the triangles that meet the given edges to coordinates in the plane.
 
     Triangle T is mapped to ((x1, y1), (x2, y2), (x3, y3)) where (xi, yi) is at the tail of side i of T when oriented anti-clockwise.
-    Coordinate are within the w x h rectangle. """
+    Coordinate are within the w x h rectangle."""
 
     r = 1000.0
 
@@ -118,7 +120,7 @@ def layout_triangulation(triangulation: "bigger.Triangulation[Edge]", components
     for component in components:
         # Create the vertices.
         num_outside = sum(1 for triangle in component for edge in triangle if edge not in interior)
-        vertices = [(r * sin(2 * pi * (i + 0.5) / num_outside), r * cos(2 * pi * (i + 0.5) / num_outside)) for i in range(num_outside)]
+        vertices = [(r * sin(2 * pi * (i - 0.5) / num_outside), r * cos(2 * pi * (i - 0.5) / num_outside)) for i in range(num_outside)]
         # Determine how many boundary edges occur between each edge's endpoints.
         # We really should do this in a sensible order so that it only takes a single pass.
         num_decendents = dict(((triangle, side), 1) for triangle in component for side in range(3) if triangle[side] not in interior)
@@ -163,15 +165,14 @@ def layout_triangulation(triangulation: "bigger.Triangulation[Edge]", components
     return layout
 
 
-def draw_block_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]", triangle: Triangle, vertices: FlatTriangle, master: int) -> None:
-    """ Draw a lamination in the given triangle as a single block. """
+def draw_block_triangle(canvas: ImageDraw, vertices: FlatTriangle, weights: List[int], master: int) -> None:
+    """ Draw a flat triangle with (blocks of) lines inside it. """
 
-    weights = [lamination(side) for side in triangle]
     weights_0 = [max(weight, 0) for weight in weights]
     sum_weights_0 = sum(weights_0)
     correction = min(min(sum_weights_0 - 2 * e for e in weights_0), 0)
     dual_weights = [bigger.half(sum_weights_0 - 2 * e + correction) for e in weights_0]
-    parallel_weights = [max(-weight, 0) for weight in weights]  # noqa: F841  # Remove once we can draw parallel things.
+    parallel_weights = [max(-weight, 0) for weight in weights]
     for i in range(3):
         # Dual arcs.
         if dual_weights[i] > 0:
@@ -191,7 +192,7 @@ def draw_block_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]",
             E1 = interpolate(vertices[i - 0], vertices[i - 1], scale_b)
             S2 = interpolate(vertices[i - 2], vertices[i - 1], scale_a2)
             E2 = interpolate(vertices[i - 0], vertices[i - 1], scale_b2)
-            draw.polygon([S1, E1, E2, S2], fill=LAMINATION_COLOUR)
+            canvas.polygon([S1, E1, E2, S2], fill=LAMINATION_COLOUR)
         elif dual_weights[i] < 0:  # Terminal arc.
             s_0 = (1 - 2 * VERTEX_BUFFER) * weights_0[i] / master
 
@@ -202,7 +203,7 @@ def draw_block_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]",
             E1 = vertices[i - 1]
             S2 = interpolate(vertices[i - 0], vertices[i - 2], scale_a2)
             E2 = vertices[i - 1]
-            draw.polygon([S1, E1, E2, S2], fill=LAMINATION_COLOUR)
+            canvas.polygon([S1, E1, E2, S2], fill=LAMINATION_COLOUR)
         else:  # dual_weights[i] == 0:  # Nothing to draw.
             pass
 
@@ -215,18 +216,17 @@ def draw_block_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]",
             MM = interpolate(SS, EE)
             s = parallel_weights[i] / master
             P = interpolate(MM, M, s)
-            draw.polygon([S, P, E], fill=LAMINATION_COLOUR)
+            canvas.polygon([S, P, E], fill=LAMINATION_COLOUR)
 
 
-def draw_line_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]", triangle: Triangle, vertices: FlatTriangle, master: int) -> None:
-    """ Draw a lamination in the given triangle as individual lines. """
+def draw_line_triangle(canvas: ImageDraw, vertices: FlatTriangle, weights: List[int], master: int) -> None:
+    """ Draw a flat triangle with (individual) lines inside it. """
 
-    weights = [lamination(side) for side in triangle]
     weights_0 = [max(weight, 0) for weight in weights]
     sum_weights_0 = sum(weights_0)
     correction = min(min(sum_weights_0 - 2 * e for e in weights_0), 0)
     dual_weights = [bigger.half(sum_weights_0 - 2 * e + correction) for e in weights_0]
-    parallel_weights = [max(-weight, 0) for weight in weights]  # noqa: F841  # Remove once we can draw parallel things.
+    parallel_weights = [max(-weight, 0) for weight in weights]
     for i in range(3):  # Dual arcs:
         if dual_weights[i] > 0:
             s_a = 1 - 2 * VERTEX_BUFFER
@@ -237,7 +237,7 @@ def draw_line_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]", t
 
                 S1 = interpolate(vertices[i - 2], vertices[i - 1], scale_a)
                 E1 = interpolate(vertices[i - 0], vertices[i - 1], scale_b)
-                draw.line([S1, E1], fill=LAMINATION_COLOUR, width=2)
+                canvas.line([S1, E1], fill=LAMINATION_COLOUR, width=2)
         elif dual_weights[i] < 0:  # Terminal arc.
             s_0 = 1 - 2 * VERTEX_BUFFER
             for j in range(-dual_weights[i]):
@@ -245,7 +245,7 @@ def draw_line_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]", t
 
                 S1 = interpolate(vertices[i - 0], vertices[i - 2], scale_a)
                 E1 = vertices[i - 1]
-                draw.line([S1, E1], fill=LAMINATION_COLOUR, width=2)
+                canvas.line([S1, E1], fill=LAMINATION_COLOUR, width=2)
         else:  # dual_weights[i] == 0:  # Nothing to draw.
             pass
 
@@ -259,30 +259,22 @@ def draw_line_triangle(draw: ImageDraw, lamination: "bigger.Lamination[Edge]", t
             for j in range(parallel_weights[i] // 2):
                 s = float(j + 1) / master
                 P = interpolate(MM, M, s)
-                draw.line([S, P, E], fill=LAMINATION_COLOUR, width=2)
+                canvas.line([S, P, E], fill=LAMINATION_COLOUR, width=2)
             if parallel_weights[i] % 2 == 1:
-                draw.line([S, E], fill=LAMINATION_COLOUR, width=2)
+                canvas.line([S, E], fill=LAMINATION_COLOUR, width=2)
 
 
-def draw_lamination(  # pylint: disable=too-many-branches
-    lamination: "bigger.Lamination[Edge]", edges: List[Edge], layout: Optional[SupportsLayout] = None, **options: Any
-) -> Image:
+def draw_lamination(lamination: bigger.Lamination[Edge], edges: List[Edge], layout: Optional[SupportsLayout] = None, **options: Any) -> Image:  # pylint: disable=too-many-branches
     """ Return an image of this lamination on the specified edges. """
-
-    if "w" not in options:
-        options["w"] = 400
-    if "h" not in options:
-        options["h"] = 400
-    if "label" not in options:
-        options["label"] = "none"
+    options = {"w": 400, "h": 400, "label": "none", "colour": "bw", "outline": False, **options}
 
     image = Image.new("RGB", (options["w"], options["h"]), color="White")
-    draw = ImageDraw.Draw(image)
+    canvas = ImageDraw.Draw(image)
 
     # Draw these triangles.
     components, interior = supporting_triangles(lamination.triangulation, edges)
     if layout is None:
-        layout2 = layout_triangulation(lamination.triangulation, components, interior)
+        layout2 = default_layout_triangulation(lamination.triangulation, components, interior)
     else:
         layout2 = dict((triangle, layout.layout(triangle)) for component in components for triangle in component)
 
@@ -320,34 +312,71 @@ def draw_lamination(  # pylint: disable=too-many-branches
             a, b, c = layout3[triangle]
             layout4[triangle] = (add(a, centre), add(b, centre), add(c, centre))
 
-    master = max(abs(lamination(edge)) for triangle in layout4 for edge in triangle)
+    # Draw triangles.
+    triangle_colours = TRIANGLE_COLOURS[options["colour"]]
     for index, (triangle, vertices) in enumerate(layout4.items()):
-        # Draw triangles.
-        draw.polygon(vertices, fill=TRIANGLE_COLOURS[index % len(TRIANGLE_COLOURS)])
+        canvas.polygon(vertices, fill=triangle_colours[index % len(triangle_colours)], outline="white" if options["outline"] else None)
 
+    weights = dict((edge, lamination(edge)) for edge in set(edge for triangle in layout4 for edge in triangle))
+
+    master = max(abs(weights[edge]) for triangle in layout4 for edge in triangle)
+    shown_is_integral = all(isinstance(weights[edge], int) for edge in weights)
+
+    # Draw lamination.
     for index, (triangle, vertices) in enumerate(layout4.items()):
-        # Draw lamination.
-        if master > MAX_DRAWABLE:
-            draw_block_triangle(draw, lamination, triangle, vertices, master)
+        if master < MAX_DRAWABLE and shown_is_integral:
+            draw_line_triangle(canvas, vertices, [weights[edge] for edge in triangle], master)
         else:  # Draw everything. Caution, this is is VERY slow (O(n) not O(log(n))) so we only do it when the weight is low.
-            draw_line_triangle(draw, lamination, triangle, vertices, master)
+            draw_block_triangle(canvas, vertices, [weights[edge] for edge in triangle], master)
 
+    # Draw labels.
     for index, (triangle, vertices) in enumerate(layout4.items()):
-        weights = [lamination(side) for side in triangle]
-        # Draw labels.
-        for i in range(3):
+        for side in range(3):
             if options["label"] == "edge":
-                text = str(triangle[i])
+                text = str(triangle[side])
             if options["label"] == "weight":
-                text = str(weights[i])
+                text = str(weights[triangle[side]])
             if options["label"] == "none":
                 text = ""
-            w, h = draw.textsize(text)
-            point = interpolate(vertices[i - 0], vertices[i - 2])
+            w, h = canvas.textsize(text)
+            point = interpolate(vertices[side - 0], vertices[side - 2])
             point = (point[0] - w / 2, point[1] - h / 2)
             for offset in OFFSETS:
-                draw.text(add(point, offset), text, fill="White", anchor="centre")
+                canvas.text(add(point, offset), text, fill="White", anchor="centre")
 
-            draw.text(point, text, fill="Black", anchor="centre")
+            canvas.text(point, text, fill="Black", anchor="centre")
 
     return image
+
+
+class DrawStructure:  # pylint: disable=too-few-public-methods
+    """ A class to record intermediate draw commands. """
+
+    def __init__(self, edges: Optional[List[Edge]] = None, **options: Any):
+        self.edges = edges
+        self.options = options
+
+    def __call__(
+        self, obj: Optional[Union[bigger.Lamination[Edge], bigger.MCG[Edge], bigger.Triangulation[Edge]]] = None, edges: Optional[List[Edge]] = None, **kwargs: Any
+    ) -> Union[DrawStructure, Image]:
+        draw_structure = DrawStructure(edges=edges if edges is not None else self.edges, **{**self.options, **kwargs})
+        if obj is None:
+            return draw_structure
+        elif draw_structure.edges is None:
+            raise TypeError("draw() missing 1 required positional argument: 'edges'")
+        elif isinstance(obj, bigger.Lamination):
+            return draw_lamination(obj, draw_structure.edges, **draw_structure.options)
+        elif isinstance(obj, bigger.Triangulation):
+            return draw_structure(obj.empty_lamination())
+        elif isinstance(obj, bigger.MCG):
+            return draw_structure(obj.triangulation)
+
+        raise TypeError("Unable to draw objects of type: {}".format(type(obj)))
+
+
+def draw(
+    obj: Optional[Union[bigger.Lamination[Edge], bigger.MCG[Edge], bigger.Triangulation[Edge]]] = None, edges: Optional[List[Edge]] = None, **options: Any
+) -> Union[DrawStructure, Image]:
+    """ Draw the given object with the provided options. """
+
+    return DrawStructure(edges=edges, **options)(obj)
