@@ -1,6 +1,6 @@
 """ A module describing common custom types used within bigger. """
 
-from typing import TypeVar, Tuple  # noqa: F401
+from typing import Tuple, TypeVar
 from typing_extensions import Protocol
 
 Edge = TypeVar("Edge")
